Attribute VB_Name = "BotUtils"
' generic bot utilities module
Option Explicit

' the following declarations allow access to some Windows API functions

Public Type Rect
    Left As Long
    Top As Long
    Right As Long
    Bottom As Long
End Type

Public Declare Function GetForegroundWindow Lib "user32" () As Long
Public Declare Function GetClassName Lib "user32" Alias "GetClassNameA" (ByVal hWnd As Long, ByVal lpClassName As String, ByVal nMaxCount As Long) As Long
Public Declare Function GetParent Lib "user32" (ByVal hWnd As Long) As Long
Public Declare Function SetParent Lib "user32" (ByVal hwndChild As Long, ByVal hwndNewParent As Long) As Long
Public Declare Function SetForegroundWindow Lib "user32" (ByVal hWnd As Long) As Boolean
Public Declare Function SetActiveWindow Lib "user32" (ByVal hWnd As Long) As Long
Public Declare Function SetWindowPos Lib "user32" (ByVal hWnd As Long, ByVal hWndInsertAfter As Long, ByVal x As Long, ByVal y As Long, ByVal cx As Long, ByVal cy As Long, ByVal wFlags As Long) As Long
Public Declare Function GetWindowRect Lib "user32" (ByVal hWnd As Long, ByRef pRect As Rect) As Boolean
Public Declare Function ShowWindow Lib "user32" (ByVal hWnd As Long, ByVal nCmdShow As Integer) As Boolean

' declare constants for use with GetWindow

Const GW_HWNDFIRST = 0
Const GW_HWNDLAST = 1
Const GW_HWNDNEXT = 2
Const GW_HWNDPREV = 3

' declare constants used with ShowWindow
Const SW_SHOW = 5

' declare constants for changing window position and Z-order

Const SWP_NOMOVE = 2
Const SWP_NOSIZE = 1
Const HWND_TOPMOST = -1
Const HWND_NOTOPMOST = -2


Function GetWebMetaInfo(sDocRoot As String) As Dictionary
    
    Dim sPath As String
    sPath = sDocRoot & "/" & "_vti_pvt/service.cnf"
    sPath = FlipSlashes(sPath)
    Set GetWebMetaInfo = GetMetaInfo(sPath)
    
End Function

Function GetPageMetaInfo(sDocRoot As String, sURL As String) As Dictionary

    Dim sPath As String
    Dim s As String
    Dim i As Integer
    Dim idx As Integer
    
    sPath = sDocRoot & "/"
    idx = InStrRev(sURL, "/")
    If idx > 0 Then
        sPath = sPath & Left(sURL, idx)
        sPath = sPath & "_vti_cnf/" & Mid(sURL, idx + 1)
    Else
        sPath = sPath & "_vti_cnf/" & sURL
    End If
    
    sPath = FlipSlashes(sPath)
    
    Set GetPageMetaInfo = GetMetaInfo(sPath)

End Function

Function GetMetaInfo(sPath As String) As Dictionary

    Dim dict As New Dictionary
    Dim fs As New FileSystemObject
    Dim f As File
    Dim s As TextStream
    Dim ln As String
    Dim idx As Integer
    Dim sKey As String
    Dim sType As String
    Dim sVal As String
    
    On Error GoTo bailout
    
    Set f = fs.GetFile(sPath)
    Set s = f.OpenAsTextStream
    While Not s.AtEndOfStream
        ln = s.ReadLine
        idx = InStr(ln, ":")
        sKey = Left(ln, idx - 1)
        ln = Mid(ln, idx + 1)
        idx = InStr(ln, "|")
        sType = Left(ln, idx - 1)
        ln = Mid(ln, idx + 1)
        sVal = ln
        If Left(sType, 1) = "V" Then
            ' vector; convert to newline-separated values,
            ' and un-escape the spaces
            sVal = Replace(sVal, "\\ ", vbTab)
            sVal = Replace(sVal, " ", vbLf)
            sVal = Replace(sVal, vbTab, " ")
        End If
        dict.Add sKey, sVal  ' ignore type
    Wend

bailout:

    Set f = Nothing
    Set s = Nothing
        
    Set GetMetaInfo = dict

End Function

Function FlipSlashes(sIn As String) As String

    FlipSlashes = Replace(sIn, "/", "\")

End Function

Function LoadFileIntoString(sPath As String) As String

    Dim s As String
    Dim ts As TextStream
    Dim fs As FileSystemObject
    Dim f As File
    
    On Error GoTo bailout
    
    Set fs = New FileSystemObject
    Set f = fs.GetFile(sPath)
    Set ts = f.OpenAsTextStream
    
    s = ts.ReadAll
    
bailout:
    
    Set ts = Nothing
    Set f = Nothing
    Set fs = Nothing
    
    LoadFileIntoString = s
    
End Function

Function GetPageAsString(sDocRoot As String, sURL As String, fJustBody As Boolean) As String

    Dim s As String
    Dim sLC As String
    Dim idxStart As Integer
    Dim idxEnd As Integer
    
    s = LoadFileIntoString(FlipSlashes(sDocRoot & "/" & sURL))
    
    If fJustBody Then
        sLC = LCase(s)
        idxStart = InStr(sLC, "<body")
        idxEnd = InStr(sLC, "</body>")
        If idxStart > 0 And idxEnd > 0 And idxEnd > idxStart Then
            idxStart = InStr(idxStart, sLC, ">")
            If idxStart > 0 Then
                s = Mid(s, idxStart + 1, idxEnd - idxStart - 1)
            End If
        End If
    End If
    
    GetPageAsString = s

End Function

Public Function GetMainFrame() As Long

    Dim ClassName As String
    ClassName = String$(255, 0)
    Dim hWnd As Long
    Dim hWndParent As Long
    Dim fpapp As FrontPage.Application
    Dim fpweb As FrontPage.WebWindow
    
    Set fpapp = New FrontPage.Application
    Set fpweb = fpapp.ActiveWebWindow
    
    fpweb.Visible = True
    fpweb.Activate
    hWnd = GetForegroundWindow()
    Do Until (hWnd = 0) Or (Left$(ClassName, GetClassName(hWnd, ClassName, Len(ClassName))) = "FrontPageExplorerWindow40")
        hWnd = GetParent(hWnd)
    Loop
    GetMainFrame = hWnd
    
    Set fpweb = Nothing
    Set fpapp = Nothing
    
End Function

Public Sub CenterFormOnFrontPage(frm As Form)

    Dim hwndFP As Long
    Dim hwndOld As Long
    Dim fWorked As Boolean
    Dim r As Rect
    
    hwndFP = GetMainFrame()
    fWorked = GetWindowRect(hwndFP, r)
    ' convert rect from pixels to twips
    r.Top = frm.ScaleX(r.Top, vbPixels, vbTwips)
    r.Left = frm.ScaleX(r.Left, vbPixels, vbTwips)
    r.Bottom = frm.ScaleX(r.Bottom, vbPixels, vbTwips)
    r.Right = frm.ScaleX(r.Right, vbPixels, vbTwips)
    frm.Move r.Left + (r.Right - r.Left - frm.Width) / 2, r.Top + (r.Bottom - r.Top - frm.Height) / 2
    
    ' form won't get displayed properly in debugger unless we do this
    If RunningInDebugger() Then
        MakeFloatingForm frm, 1
    End If

End Sub

Sub MakeFloatingForm(f As Form, yesno As Integer)

    Dim ret As Long

    If yesno Then
        ret = SetWindowPos(f.hWnd, HWND_TOPMOST, 0, 0, 0, 0, SWP_NOMOVE Or SWP_NOSIZE)
    Else
        ret = SetWindowPos(f.hWnd, HWND_NOTOPMOST, 0, 0, 0, 0, SWP_NOMOVE Or SWP_NOSIZE)
    End If

End Sub

Function RunningInDebugger() As Boolean

    RunningInDebugger = False
 
    On Error Resume Next
    ' The Debug object is not used in compiled code
    Err.Clear
    Debug.Print 0 / 0
    If Err.Value <> 0 Then RunningInDebugger = True

End Function
