//
// Copyright (c) 1999 Microsoft Corporation
// votecount.cpp : 
//  CVote implementation

#include "votecount.h"
#include <string.h>
#include <stdlib.h>


#ifdef WIN32
#include <direct.h>
#include <windows.h>
#include <io.h>
#include <share.h>
#endif 

#include <errno.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdio.h>

///////////////////////////////////////////////////////////////////
CVote::CVote(const char *filename, BOOL create, const char* sVoteKey1, const char* sVoteKey2)
{
    
	m_VoteType1 = new char[15];
	m_VoteType2 = new char[15];
	strcpy(m_VoteType1, sVoteKey1);
	strcpy(m_VoteType2, sVoteKey2);
    
	MakeWritable(filename);
	
	//Open the file and lock it.  Timeout for waiting for lock is 10 sec
    if (!OpenAndLock(filename, 10, create))    
       return;
	
	error = FALSE;
}

///////////////////////////////////////////////////////////////////
CVote::~CVote()
{
    Close();            // Closes and unlocks the file    
	delete m_VoteType1;
	delete m_VoteType2;
}


///////////////////////////////////////////////////////////////////
void CVote::MakeWritable(const char *filename)
{
    chmod(filename, _S_IREAD | _S_IWRITE);
}

///////////////////////////////////////////////////////////////////
BOOL CVote::OpenAndLock(const char *filename, int nTimeoutSec, BOOL create)
{

    DWORD startTime = GetTickCount();
    DWORD nTimeout = nTimeoutSec * 1000;        // Timeout in milleseconds
    for (fd = -1; nTimeoutSec == 0 || GetTickCount() - startTime < nTimeout;)
    {
        // Try to open the file in locked mode 
        fd = _sopen(filename, _O_RDWR, _SH_DENYWR);

        // Success, exit the for loop
        if (fd != -1)
            break;

        // No success. If file was not found, and create is specified, then
        // create it, lock it, and initialize it
        if (errno == ENOENT && create)
        {
            fd = _sopen(filename, _O_RDWR | _O_CREAT, _SH_DENYWR, _S_IREAD | _S_IWRITE);
			InitializeFile();
            break;
        }

        // Otherwise, delay half a second, and try again.
        Sleep(500);
    }
    if (fd == -1)
        return FALSE;        // Timed out

    return TRUE;
}

///////////////////////////////////////////////////////////////////
void CVote::InitializeFile()
{  
	// called whenever the votefile needs to be created from scratch
	// (when file is first saved w/ votebot on it, or when a recalc
	// links occurs and .vot file isn't found, or when voteopt has
	// changed via the bot's edit method
	lseek(fd, 0, SEEK_SET);
	
	write (fd, "\n", 1);    // file descriptor line
	
	write (fd, this->m_VoteType1, strlen(this->m_VoteType1));
	write (fd, ":0", 2);
	write (fd, "\n", 1);
	write(fd, this->m_VoteType2, strlen(this->m_VoteType2));
	write(fd, ":0", 2);
	
};

///////////////////////////////////////////////////////////////////
void CVote::Close()
{
	// close the .vot file
    close(fd);
}


///////////////////////////////////////////////////////////////////
BOOL CVote::IsValid()
{
	// determine whether or not .vot file is a valid one.
	// if it's corrupted, calling function should create a new one.
    lseek(fd, 0, SEEK_SET);
    char buffer[KEY_VALUE_LEN];
    int nBytesRead = read(fd, buffer, KEY_VALUE_LEN);
    if (nBytesRead == -1)
    {
        error = TRUE;
        return FALSE;
    }
    return (strncmp(buffer, KEY1_STR, KEY_VALUE_LEN) == 0);
}
