//
// Copyright (c) 1999 Microsoft Corp.  All Rights Reserved

// CCount class
//
// Encapsulates the locking, reading, incrementing, and unlocking of
// the vote count data file


#define DIR_SEPARATOR '\\'

typedef int BOOL;

#ifndef FALSE
#define FALSE 0
#endif

#ifndef TRUE
#define TRUE 1
#endif

#include "webbot.h"

class CVote
{
    // Public Methods
public:
    // The constructor opens and locks the file, with a timeout of 10
    // seconds if the file is already locked by another process.
    // If the file doesn't exist, and "create" is true, then it 
    // creates the count file with a count of 0.
    CVote(const char *filename, BOOL create, const char* sVoteKey1, const char* sVoteKey2);
    
    ~CVote();                        // closes and unlocks the file

	const char *votetype;

	void InitializeFile();           // Initializes File w/ votetype keys.

    // Private Data Members
private:
    char* m_VoteType1;		// for this object (file), the first vote option
	char* m_VoteType2;		// for this object (file), the second vote option
    int fd;
    BOOL error;
    BOOL OpenAndLock(const char *filename, int nTimeoutSec, BOOL create);
    void Close();                       // closes .vot file, called by destructor
    void MakeWritable(const char *filename);   // .vot file prep
    BOOL IsValid();                            // checks .vot file for validity

};


// Keys to identify a vote count file
#define KEY1_STR "vote1type"
#define KEY2_STR "vote2type"
#define KEY_VALUE_LEN 1
#define VOTE_NAME_LEN 10

