//
// Copyright (c) 1999 Microsoft Corp.  All Rights Reserved

// votefile.cpp : 
// 1) Routines that handle the path/file creation for server extensions.
// 
#include "votecount.h"
#include "webbot.h"
#include <string.h>
#include <stdlib.h>
#include <assert.h>

#ifdef WIN32
#include <direct.h>
#include <windows.h>
#include <io.h>
#include <share.h>
#endif 

#include <errno.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdio.h>

const int URLNumTranslations = 2;
const char URLTranslateFrom[URLNumTranslations] = {' ', '\\'};

////////////////////////////////////////////////////////////////////////
// Utility URL and filename manipulation functions.

// Extracts just the filename portion of the URL.
// that is, a URL of "/simple/foo/default.htm" becomes "default.htm",
// and "blah.html" would become "blah.html"

// TODO: remove this.  It's not needed for votebot2
const char* URLtoFileName(const char *URL)
{
    if (!URL)
        return "";

    // Look for the last '/' in the URL:
    const char *last = URL;
    for (const char *p=URL; *p; p++)
        if (*p == '/')
            last = p+1;
    return last;
}

///////////////////////////////////////////////////////////////////
char *TranslateChar(const char *PageURL, char translateFROMChar, char translateTOChar)
{
	// returns a string that is PageURL w/ every occurrence of translateFROMChar replaced
	// with translateTOChar
	// users of this function should delete the pointer after use

    if (!PageURL)
        return "";
    
	char *pTranslatedString = new char[strlen(PageURL) + 1]; // + 1 for \0
	char *pTransStringloc = pTranslatedString;
	
    for (; *PageURL; PageURL++, pTransStringloc++)
    {
        if (*PageURL == translateFROMChar) {*pTransStringloc = translateTOChar; }
		else { *pTransStringloc = *PageURL; }
	}

	*pTransStringloc = '\0';

	return pTranslatedString;
}

///////////////////////////////////////////////////////////////////
const char *ReturnBaseFolder(const char* sFilePath)
{
    // returns a URL that equals sPageURL with everything after the trailing / snipped.
  	// users of this function should delete the pointer after use
	
	if (!sFilePath)
        return "";
    
	const char* last = strrchr(sFilePath,'\\'); 

	if (last == NULL)
		return "";

	char* newstring = new char[(last-sFilePath)+1];
	strncpy (newstring, sFilePath, last-sFilePath); 	
	newstring[(last-sFilePath)]= '\0';  // close off newstring
	return newstring;
}

///////////////////////////////////////////////////////////////////
const char *mkdirNested(const char* sPageURL)
{
	if (access(ReturnBaseFolder(sPageURL), 0) != 0) { return sPageURL; }
	else 
	{	mkdirNested(ReturnBaseFolder(sPageURL)); 
		mkdir(sPageURL);
		return sPageURL;
	}

}

///////////////////////////////////////////////////////////////////
// Vote count file functions

void CreateVoteResultsFile(const char *VoteCntFile, char* sVoteType1, char* sVoteType2)
{
    // Create a CVote object for dealing with the vote count file.
    // Waits until the file is available, then locks it.  Times out
    // if the file does not become available in a certain amount of time.
    // If the file does not exist, it will create it.
   
	CVote votefile(VoteCntFile, TRUE, sVoteType1, sVoteType2);

    // CVote destructor writes and unlocks the file
}


void HandleVoteCntFile(CWebBotDict &bot, char* sVoteType1, char* sVoteType2)
{    
    // Figures out the filename for the vote count results file.  
    // and then creates it and the directory structure under _private
	// if it doesn't exist.

   	// VoteFilePath = sDocumentRoot + "\_private\" + TranslateChar(sPathInfo) + ".vot"
    
	const char *bReformat = bot.GetValue("B-REFORMAT");
	const char *PageURL = bot.GetValue("PageURL");
    const char *DocumentRoot = bot.GetValue("DocumentRoot");
    const char *BotID = bot.GetValue("BOTID");
	
    char* TranslatedString = TranslateChar(PageURL,'/','\\');

    if (!PageURL || !DocumentRoot)
        return;

    // Create a buffer large enough:
    int DocumentRootLen = strlen(DocumentRoot);
    
	char *PageDirName = new char[DocumentRootLen  
                            + 10    +                  // +10 for "\_private\"
                            strlen(PageURL) +          // +1 for DIR_SEPARATOR
                            strlen(BotID) +           
							+ 5];    // +5 for ".vot"+'\0'
        
	
    // Construct the .vot path:
    strcpy(PageDirName, DocumentRoot);
	strcat(PageDirName, "\\_private\\");
	strcat(PageDirName, TranslatedString);
	strcat(PageDirName, BotID);
	strcat(PageDirName, ".vot");
	
	bot.SetValue("U-votepath",PageDirName);
    
	//Check for existence of, and create directory
    if (access(PageDirName, 0) != 0) 
    {	mkdirNested(ReturnBaseFolder(PageDirName));    }
    
    ////////////////////////////////////////////////////
    // Now buffer holds the filename of the vote count file.

    if (access(PageDirName, 0) != 0)
	{	CreateVoteResultsFile(PageDirName, sVoteType1, sVoteType2);	}
	else if (!strcmp(bReformat,"TRUE"))		
	{	CreateVoteResultsFile(PageDirName, sVoteType1, sVoteType2);
		bot.SetValue("B-REFORMAT", "FALSE");	}
		
    
	delete[] PageDirName; 
	delete TranslatedString;

}

char *URLEncode(const char *url, const char *translateFrom, const char **translateTo, 
                int numTranslations)
{
    // Note: Users of this function should delete the pointer themselves
    if (url == NULL)
        return NULL;

    // maximum size of the encoded string is the url's len*3.  This assumes that
    // no TranslateTo string is more than 3 chars, which will always be
    // true for URL encoding
    char *encoded = new char[strlen(url)*3 + 1];            // +1 for '\0'

    char *currentTo = encoded;
    BOOL found;
    for (const char *currentFrom = url; *currentFrom; currentFrom++)
    {
        found = FALSE;
        for (int i = 0; i < numTranslations && !found; i++)
        {
            if (*currentFrom == translateFrom[i])
            {
                // Matches a char, copy over the translateTo for this char
                strcpy(currentTo, translateTo[i]);
                currentTo+=strlen(translateTo[i]);
                found = TRUE;
            }
        }

        if (!found)
        {
            // Wasn't a char to translate, just do plain vanilla copy
            *currentTo = *currentFrom;
            currentTo++;
        }
    }
    *currentTo = '\0';    // Close it off
    return encoded;
}


