//
// Copyright (c) 1999 Microsoft Corp.  All Rights Reserved

// setupDlg.cpp : implementation file
//

#include "stdafx.h"
#include "setup.h"
#include "setupDlg.h"
#include "fileutil.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


/////////////////////////////////////////////////////////////////////////////
// CSetupDlg dialog

CSetupDlg::CSetupDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CSetupDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CSetupDlg)
	m_szExtractDir = _T("");
	m_InstallType = -1;
	m_ComponentType = -1;
	//}}AFX_DATA_INIT
	// Note that LoadIcon does not require a subsequent DestroyIcon in Win32
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);

	m_versionASP = TRUE;

	char buffer[MAX_PATH];

	// Look for the temp directory:
	if (GetEnvironmentVariable("TEMP", buffer, MAX_PATH) !=0)
		m_szExtractDir = CString(buffer) + "\\fpvoter";
	else if (GetEnvironmentVariable("TMP", buffer, MAX_PATH) !=0)
		m_szExtractDir = CString(buffer) + "\\fpvoter";
	else if (GetEnvironmentVariable("TMPDIR", buffer, MAX_PATH) !=0)
		m_szExtractDir = CString(buffer) + "\\fpvoter";
	else
		m_szExtractDir = "c:\\temp\\fpvoter";

}

void CSetupDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CSetupDlg)
	DDX_Text(pDX, IDC_EXTRACT_DIR, m_szExtractDir);
	DDX_Radio(pDX, IDC_RADIO1, m_InstallType);
	DDX_Radio(pDX, IDC_AspVersion, m_ComponentType);
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CSetupDlg, CDialog)
	//{{AFX_MSG_MAP(CSetupDlg)
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	ON_BN_CLICKED(IDC_RADIO1, OnInstallType)
	ON_BN_CLICKED(IDC_RADIO2, OnInstallType)
	ON_BN_CLICKED(IDC_AspVersion, OnComponentType)
	ON_BN_CLICKED(IDC_CgiVersion, OnComponentType)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CSetupDlg message handlers

BOOL CSetupDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	// Set the icon for this dialog.  The framework does this automatically
	//  when the application's main window is not a dialog
	// SetIcon(m_hIcon, TRUE);			// Set big icon
	// SetIcon(m_hIcon, FALSE);		// Set small icon
	

	/////////////////////////////////////////////////////////////////////
	// Look for MS IIS-based servers

	m_iisInstallable = m_iisServer.IsInstallable();

	if (m_iisInstallable || m_pwsInstallable)
	{
		GetDlgItem(IDC_RADIO2)->EnableWindow(TRUE);
		m_InstallType = 1;
	}
	else
	{
		GetDlgItem(IDC_RADIO2)->EnableWindow(FALSE);
		m_InstallType = 0;
	}

	m_ComponentType = 0;

	UpdateData(FALSE);
	OnInstallType();

	return TRUE;  // return TRUE  unless you set the focus to a control

}

// If you add a minimize button to your dialog, you will need the code below
//  to draw the icon.  For MFC applications using the document/view model,
//  this is automatically done for you by the framework.

void CSetupDlg::OnPaint() 
{
	if (IsIconic())
	{
		CPaintDC dc(this); // device context for painting

		SendMessage(WM_ICONERASEBKGND, (WPARAM) dc.GetSafeHdc(), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialog::OnPaint();
	}
}

// The system calls this to obtain the cursor to display while the user drags
//  the minimized window.
HCURSOR CSetupDlg::OnQueryDragIcon()
{
	return (HCURSOR) m_hIcon;
}

void CSetupDlg::OnOK() 
{
	UpdateData();

	CString installMsg;
	if (m_InstallType == 0)		// Manual installation
	{
		if (!IsDir(m_szExtractDir))
		{
			CString errMsg;
			AfxFormatString1(errMsg, IDS_NOT_DIR, m_szExtractDir);
			if (AfxMessageBox(errMsg, MB_YESNO) == IDYES)
			{
				if (!FullyCreateDirectory(m_szExtractDir))
					return;
			}
			else
				return;
		}
		
		// Copy the files
		if (!m_versionASP)
		{
			if ((MyCopyFile(".\\votebot.inf", m_szExtractDir)) &&	
			(MyCopyFile(".\\votebot.exe", m_szExtractDir)) &&
			(MyCopyFile(".\\votebot.dll", m_szExtractDir)) &&
			(MyCopyFile(".\\votesrvr.dll", m_szExtractDir)))
			//todo: Add?? MyCopyFile("readme.txt", m_szExtractDir);
			{	installMsg.LoadString(IDS_INSTALL_ALL);		
				installMsg.FormatMessage(installMsg, m_szExtractDir);	}
			else
			{	installMsg.LoadString(IDS_EXTRACT_ERROR);		
				installMsg.FormatMessage(installMsg, m_szExtractDir);	}
		}	/* End If (ComponentType == CGI) */
		else
		{
			if ((MyCopyFile(".\\aspvoter.dll", m_szExtractDir)) &&
			(MyCopyFile(".\\aspvoter.inf", m_szExtractDir)))
			{	installMsg.LoadString(IDS_INSTALL_ALL);		
				installMsg.FormatMessage(installMsg, m_szExtractDir);	}
			else
			{	installMsg.LoadString(IDS_EXTRACT_ERROR);
				installMsg.FormatMessage(installMsg, m_szExtractDir);	}
		}   /* End If (InstallType == ASP) */
	}
	else			// Automatic installation
	{
		BOOL success = TRUE;
		if (m_iisInstallable)
			success &= m_iisServer.Install(m_versionASP);	
		if (success)
			installMsg.LoadString(IDS_INSTALL_OTHER);
		else
		{
			AfxMessageBox(IDS_INSTALL_ERROR);
			m_InstallType = 0;			
			UpdateData(FALSE);
			OnInstallType();
			return;
		}
	}

	ShowWindow(SW_HIDE);

	AfxMessageBox(installMsg);
	
	CDialog::OnOK();
}

void CSetupDlg::OnInstallType() 
{
	UpdateData();
	GetDlgItem(IDC_EXTRACT_DIR)->EnableWindow(m_InstallType == 0);	
}


void CSetupDlg::OnComponentType() 
{
	UpdateData();
	m_versionASP = !m_ComponentType;
	
}
